# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that show the customized tooltip for
the status area.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Window(object):
	"""
	This class shows the tooltip window for the status area.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__change_bgcolor()
		self.__precompile_methods()
		self.__signal_id_1 = manager.connect("show-status-window", self.__show_status_window_cb)
		self.__signal_id_2 = manager.connect("hide-status-window", self.__hide_status_window_cb)
		self.__signal_id_3 = manager.connect("icon-position", self.__icon_position_cb)
		self.__signal_id_4 = manager.connect("destroy", self.__destroy_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__manager = manager
		self.__player = player
		self.__screen = None
		self.__orientation = None
		self.__position = None
		self.__is_visible = True
		self.__window = manager.glade.get_widget("StatusWindow")
		return

	def __change_bgcolor(self):
		from gtk import STATE_NORMAL
		from gtk.gdk import color_parse
		style = self.__window.get_style().copy()
		color = color_parse("#aaaaaa")
		style.bg[STATE_NORMAL] = color
		self.__window.set_style(style)
		return

	def __show_status_window_cb(self, *args):
		"""
		Handles callback when the "show-status-window" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__show()
		return

	def __hide_status_window_cb(self, *args):
		"""
		Handles callback when the "hide-status-window" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__hide()
		return

	def __icon_position_cb(self, manager, data):
		"""
		Handles callback when the "icon-position" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param data: Status icon position and related data.
		@type data: A Tuple object.
		"""
		from operator import not_
		if not_(data): return
		self.__screen, self.__rectangle, self.__orientation = data
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__manager)
		disconnect_signal(self.__signal_id_3, self.__manager)
		disconnect_signal(self.__signal_id_4, self.__manager)
		self.__window.destroy()
		delete_attributes(self)
		del self
		self = None
		return

	def __show(self):
		"""
		Show the window.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		if self.__is_visible: return
		self.__player.response()
		self.__position_window()
		self.__window.show_all()
		self.__player.response()
		self.__is_visible = True
		return

	def __hide(self):
		"""
		Hide window.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		from operator import not_
		if not_(self.__is_visible): return
		self.__player.response()
		self.__window.hide()
		self.__player.response()
		self.__is_visible = False
		return

	def __position_window(self):
		"""
		Determine where to position window.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		x = self.__calculate_x_cordinate()
		y = self.__calculate_y_cordinate()
		self.__window.move(x, y)
		return

	def __calculate_x_cordinate(self):
		"""
		The vertical coordinate to move the window to.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@return: The vertical coordinate to move window to.
		@rtype: An Integer object.
		"""
		min_x = self.__rectangle.x
		window_width = self.__window.get_size()[0]
		max_x = min_x + window_width
		min_root_x = 0
		max_root_x = self.__screen.get_width()
		position = min_x - ((max_x - min_x) / 2)
		from operator import gt
		if gt((position + window_width), max_root_x):
			position = position - ((position + window_width) - max_root_x)
		if gt(min_root_x, position):
			position = min_root_x
		return position

	def __calculate_y_cordinate(self):
		"""
		The vertical coordinate to move the window to.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@return: The vertical coordinate to move window to.
		@rtype: An Integer object.
		"""
		top_panel = 0 + self.__rectangle.height
		min_y = self.__rectangle.y
		from operator import contains
		top_expression = contains(range(top_panel-5, top_panel+5), self.__rectangle.y)
		if top_expression:
			position = self.__rectangle.y + self.__rectangle.height + 3
		else:
			position = self.__rectangle.y - self.__window.get_size()[1] - 3
		return position

	def __precompile_methods(self):
		"""
		Use psyco to optimize methods.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		try:
			from psyco import bind
			bind(self.__show)
			bind(self.__position_window)
			bind(self.__calculate_x_cordinate)
			bind(self.__calculate_y_cordinate)
		except ImportError:
			pass
		return
