# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the a class that manages the volume controls.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_FLOAT, TYPE_BOOLEAN

class Manager(GObject):
	"""
	This class manages the volume control widgets.
	"""

	__gsignals__ = {
		"destroy": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"increase-volume": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"decrease-volume": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"volume": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_FLOAT, TYPE_BOOLEAN))
	}

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		GObject.__init__(self)
		self.__init_attributes(player)
		from VolumeRange import VolumeRange
		VolumeRange(self, player.glade, player.feedback)
		from VolumeUpButton import VolumeUpButton
		VolumeUpButton(self, player.glade)
		from VolumeDownButton import VolumeDownButton
		VolumeDownButton(self, player.glade)
		from gnomevfs import monitor_add, MONITOR_FILE
		self.__volume_id = monitor_add(self.__volume_database_uri,
									MONITOR_FILE,
									self.__database_changed_cb)
		self.__sig_id_1 = self.connect("destroy", self.__destroy_cb)
		self.__sig_id_2 = player.connect_after("streamer-is-ready", self.__streamer_is_ready_cb)
		self.__activate_widgets()

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		from Metadata import get_volume_from_database
		self.__volume = get_volume_from_database()
		# Path to the volume database.
		from Striim.Global import metadata_folder
		self.__volume_database_path = metadata_folder + "/volume.gdb"
		from gnomevfs import get_uri_from_local_path
		self.__volume_database_uri = get_uri_from_local_path(self.__volume_database_path)
		self.__volume_id = None
		return

	def __activate_widgets(self):
		"""
		Announce volume level to volume widgets.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from operator import not_
		if not_(self.__player.is_ready): return
		self.emit("volume", self.__volume, True)
		return

	def __streamer_is_ready_cb(self, *args):
		"""
		Handles callback when the streamer is ready.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__activate_widgets()
		return

	def __broadcast_volume_level(self):
		"""
		Announce new volume level to all volume widgets.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Metadata import get_volume_from_database
		self.__volume = get_volume_from_database()
		self.emit("volume", self.__volume, False)
		return

	def __database_changed_cb(self, monitor_uri, info_uri, event_type):
		"""
		Handles callback when the volume database is modified.

		@param self: Reference to the Manager instance.
		@type self: An Manager object.

		@param monitor_uri: The uri that is monitored.
		@type monitor_uri: A String object.

		@param info_uri: The uri that is monitored.
		@type info_uri: A String object.

		@param event_type: The type of modification that occured.
		@type event_type: A gnomevfs.MONITOR_EVENT* object.
		"""
		from gnomevfs import MONITOR_EVENT_DELETED
		from gnomevfs import MONITOR_EVENT_CREATED
		from gnomevfs import MONITOR_EVENT_CHANGED
		if event_type in [MONITOR_EVENT_DELETED, MONITOR_EVENT_CREATED, MONITOR_EVENT_CHANGED]:
			from gobject import idle_add, source_remove
			try:
				source_remove(self.__broadcast_volume_level_id)
			except:
				pass
			self.__broadcast_volume_level_id = idle_add(self.__broadcast_volume_level)
		return

	def destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.emit("destroy")
		return

	def increase_volume(self):
		"""
		Increase player volume.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.emit("increase-volume")
		return

	def decrease_volume(self):
		"""
		Decrease player volume.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.emit("decrease-volume")
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import disconnect_signal, delete_attributes
		disconnect_signal(self.__sig_id_1, self)
		disconnect_signal(self.__sig_id_2, self.__player)
		if self.__volume_id:
			from gnomevfs import monitor_cancel
			monitor_cancel(self.__volume_id)
		delete_attributes(self)
		del self
		self = None
		return
